"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketClientTransport = void 0;
const transport_1 = require("ataraxia/transport");
const WebSocketClientPeer_1 = require("./WebSocketClientPeer");
/**
 * Transport that connects to the network over a web socket. This type of
 * transport requires a URL to connect to and the authentication methods to
 * use.
 *
 * For use outside browsers a function that creates a `WebSocket` instance
 * may be provided.
 *
 * Example:
 *
 * ```javascript
 * import { Network, AnonymousAuth } from 'ataraxia';
 * import { WebSocketClientTransport } from 'ataraxia-ws-client';
 *
 * // Setup a network with a WebSocket client
 * const net = new Network({
 *   name: 'name-of-your-app-or-network',
 *
 *   transports: [
 *
 *     new WebSocketClientTransport({
 *       // URL to the websocket on the server
 *       url: 'ws://localhost:7000',
 *       // If using outside a browser, define how a WebSocket is created
 *       factory: url => new WebSocket(url),
 *       // Use anonymous authentication
 *       authentication: [
 *         new AnonymousAuth()
 *       ]
 *     })
 *
 *   ]
 * });
 *
 * await net.start();
 * ```
 */
class WebSocketClientTransport extends transport_1.AbstractTransport {
    constructor(options) {
        super('ws-client');
        this.options = options;
    }
    async start(options) {
        const started = await super.start(options);
        if (!started)
            return false;
        const factory = this.options.webSocketFactory
            || defaultWebSocketFactory;
        this.peer = new WebSocketClientPeer_1.WebSocketClientPeer(this.network, this.options.authentication, factory, this.options.url);
        this.addPeer(this.peer);
        // Start connecting to the peer
        this.peer.tryConnect();
        return true;
    }
    async stop() {
        const stopped = await super.stop();
        if (!stopped)
            return false;
        if (this.peer) {
            this.peer.disconnect();
        }
        return true;
    }
}
exports.WebSocketClientTransport = WebSocketClientTransport;
/**
 * Default factory for creating a `WebSocket`. Will look for a global variable
 * called `WebSocket` which works well in browsers.
 *
 * @param url -
 *   URL to connect to
 * @returns
 *   instance of `WebSocket`
 */
function defaultWebSocketFactory(url) {
    if (typeof WebSocket !== 'object') {
        throw new Error('No default WebSocket implementation found');
    }
    return new WebSocket(url);
}
//# sourceMappingURL=WebSocketClientTransport.js.map