import { AbstractPeer, PeerMessageType, encodeBinaryPeerMessage, decodeBinaryPeerMessage, DisconnectReason } from 'ataraxia/transport';
/**
 * Abstract implementation of a peer suitable for use with a WebSocket
 * instance.
 */
export class AbstractWebSocketPeer extends AbstractPeer {
    constructor(parent, authProviders) {
        super(parent, authProviders);
    }
    setSocket(socket) {
        this.socket = socket;
        socket.binaryType = 'arraybuffer';
        socket.addEventListener('message', event => {
            if (!(event.data instanceof ArrayBuffer)) {
                // We only work with ArrayBuffer
                return;
            }
            const decoded = decodeBinaryPeerMessage(event.data);
            if (decoded) {
                this.receiveData(decoded[0], decoded[1]);
            }
        });
        socket.addEventListener('error', () => {
            if (socket === this.socket) {
                this.handleDisconnect(DisconnectReason.Error, new Error('Disconnecting due to error'));
            }
        });
        socket.addEventListener('close', () => {
            if (this.socket) {
                // Unknown disconnect reason
                this.handleDisconnect(DisconnectReason.Error);
            }
        });
    }
    requestDisconnect(reason, err) {
        if (!this.socket) {
            return;
        }
        const socket = this.socket;
        this.handleDisconnect(reason, err);
        socket.close();
    }
    disconnect() {
        super.disconnect();
        if (!this.socket) {
            return;
        }
        const socket = this.socket;
        this.handleDisconnect(DisconnectReason.Manual);
        socket.close();
    }
    handleDisconnect(reason, err) {
        this.socket = undefined;
        super.handleDisconnect(reason, err);
    }
    async send(type, payload) {
        if (!this.socket) {
            this.debug('No socket but tried to send', type, 'with data', payload);
            return;
        }
        this.debug('Sending', PeerMessageType[type], 'with data', payload);
        const data = encodeBinaryPeerMessage(type, payload);
        this.socket.send(data);
    }
}
//# sourceMappingURL=AbstractWebSocketPeer.js.map