import { AuthProvider } from 'ataraxia';
import { AbstractTransport, TransportOptions } from 'ataraxia/transport';
import { WebSocketFactory } from './WebSocketFactory';
/**
 * Options that can be used with `WebSocketClientTransport`.
 */
export interface WebSocketClientTransportOptions {
    /**
     * The URL to the websocket.
     */
    url: string;
    /**
     * Factory used to create a WebSocket.
     */
    webSocketFactory?: WebSocketFactory;
    /**
     * Authentication providers to use for this transport.
     */
    authentication: ReadonlyArray<AuthProvider>;
}
/**
 * Transport that connects to the network over a web socket. This type of
 * transport requires a URL to connect to and the authentication methods to
 * use.
 *
 * For use outside browsers a function that creates a `WebSocket` instance
 * may be provided.
 *
 * Example:
 *
 * ```javascript
 * import { Network, AnonymousAuth } from 'ataraxia';
 * import { WebSocketClientTransport } from 'ataraxia-ws-client';
 *
 * // Setup a network with a WebSocket client
 * const net = new Network({
 *   name: 'name-of-your-app-or-network',
 *
 *   transports: [
 *
 *     new WebSocketClientTransport({
 *       // URL to the websocket on the server
 *       url: 'ws://localhost:7000',
 *       // If using outside a browser, define how a WebSocket is created
 *       factory: url => new WebSocket(url),
 *       // Use anonymous authentication
 *       authentication: [
 *         new AnonymousAuth()
 *       ]
 *     })
 *
 *   ]
 * });
 *
 * await net.start();
 * ```
 */
export declare class WebSocketClientTransport extends AbstractTransport {
    private readonly options;
    private peer?;
    constructor(options: WebSocketClientTransportOptions);
    start(options: TransportOptions): Promise<boolean>;
    stop(): Promise<boolean>;
}
//# sourceMappingURL=WebSocketClientTransport.d.ts.map